### INCOMPLETE ###
import numpy as np
import pandas as pd


def generate_rashomon_set(models: np.array,
                          data: pd.DataFrame,
                          metric: callable,
                          epsilon: float = 0.2) -> set:
    """
    Genearte a Rashomon set using trained models by first finding the optimal model given a metric and data, 
    then adding every model performing within parameter epsilon worse than optimal on the same metric.

    Parameters:
        models: Array of fitted models that could be eligible for the Rashomon set.
        data: Data frame which will be used to evalute models in a metric.
        metric: Callable function that accepts (model, data) as parameters and returns a float score tending to 0.
        epsilon: Hyperparameter to determine acceptable error.
    
    Returns:
        Set containing models that fit the criteria of a Rashomon set.

    Example use:
        TO-DO!
    """
    scores = np.array([metric(model, data) for model in models])
    optimal_score = np.min(scores)
    model_in_set = scores <= optimal_score * (1 + epsilon)
    rashomon_set = set(models[model_in_set])
    return rashomon_set


def pdp(model, 
        data: pd.DataFrame, 
        feature: str, 
        feature_value: float) -> float:
    """
    Given a model for a predictive function, modify a specified feature to a given value in a copy of data to 
    calculate the mean over all data point variations.

    Parameters:
        model: Model that will be used for the prediction function.
        data: Data frame which will be copied and have a specific feature uniformally modified for PDP.
        feature: Specific feature column name in the data to modify.
        feature_value: Desired feature value for PDP calculation.

    Returns:
        Float value for the Partial Dependence Profile (PDP)

    Example use:
        TO-DO!
    """
    # Prepare calculation
    if feature not in data.columns:
        raise KeyError("PDP feature does not exist in given data.")
    modified_data = data.copy()
    modified_data[feature] = feature_value
    results = np.zeros(len(data))

    # Calculate PDP of given feature value
    for idx, sample in enumerate(modified_data):
        results[idx] = model.predict(sample)
    return np.mean(results)


def rashomon_pdp(models: np.array,
                 data: pd.DataFrame,
                 feature: str,
                 feature_value: float) -> float:
    """
    !!! INCOMPLETE !!!
    Description to be added based on the functionality and paper.
    Need to include Confidence Interval calculation here or in next function.

    Parameters:
        models: Rashomon set to be used in calculating the unified central trend.
        data: Data frame which be used to generate the PDPs in the models.
        feature: Specific feature column name in the data to modify.
        feature_value: Desired feature value for PDP calculation.
    
    Returns:
        Mean of PDPs from the Rashomon set models.
    
    Example use:
        TO-DO!
    """
    pdp_values = np.array([pdp(model, data, feature, feature_value) for model in models])
    return np.mean(pdp_values)  # Requires bootstrapping before completion



"""
A file where we can create functions that will render different types of plots
Author: Move 78
Date: 17-03-2026
"""

import seaborn as sns
import matplotlib.pyplot as plt
import pandas as pd

def ci_plot(data: pd.DataFrame,
                      x_col: str,
                      y_col: str,
                      x_title: str,
                      y_title: str,
                      title: str,
                      description: str) -> None:
    """
    Plots a Seaborn line plot with confidence interval that contains a title and a sub-plot description.
    
    Parameters:
        data (float): The data for the plot.

        x_col (str): Give the name of the x-axis column.
        y_col (str): Give the name of the y-axis column.

        x_title (str): Give the title of the x-axis.
        y_title (str): Give the title of the y-axis.

        title (str): Give the title of the plot.
        description (str): Put a small description of the plot.

    Returns:
        Renders a plot.
    """

    # Choose the plot size and the type of plot.
    plt.figure(figsize=(10, 6))
    ax = sns.lineplot(data=data, x=x_col, y=y_col, errorbar=('ci', 95))

    # Add Title and Labels.
    ax.set_title(title, fontsize=16, fontweight='bold', pad=20)
    ax.set_xlabel(x_title.capitalize(), fontsize=12)
    ax.set_ylabel(y_title.capitalize(), fontsize=12)

    # Add Description below the plot.
    plt.figtext(0.5, 0.01, description, wrap=True, horizontalalignment='center',color='gray')

    # Render the plot.
    plt.show()


def example_plot():
    """
    A test function to use the ci_plot function

    Returns:
        Nothing on its own
    """
    # A data set from seaborn.
    df = sns.load_dataset("dots")
    # A call to the ci_plot function with all the parameters.
    ci_plot(df, "time", "firing_rate", "surface_area", "Response", "Rashomon Set- PDP visualisation",
            "Note: The shaded area represents the 95% confidence interval.")


if __name__ == "__main__":
    example_plot()



"""
The first version of the Rashomon Toolkit.
Author: Move 78
Date: 02-03-2026
"""
import matplotlib.pyplot as plt

def add(x,y):
    """Just a simple add function"""
    return x + y

def get_name():
    """Function that returns the package name"""
    return "Rashomon Toolkit"

def plot_test_graph():
    """
    Creates a simple line graph to test other libraries.
    And how it behaves with pip
    """
    # Some data to plot
    x = [0, 1, 2, 3, 4, 5]
    y = [0, 1, 4, 9, 16, 25] 

    plt.figure(figsize=(8, 5))
    plt.plot(x, y, marker='o', linestyle='-', color='b', label='Growth')
    
    # Adding titles and labels
    plt.title("Simple Square Growth Graph")
    plt.xlabel("Input Value (x)")
    plt.ylabel("Squared Value (y)")
    plt.legend()
    plt.grid(True)
    
    plt.show()

class Toolkit: 
    def __init__(self,owner="Guest"):
        self.owner = owner
    
    def hello(self):
        """Welcomes the user to the toolkit"""
        return f"Hello {self.owner} and welcome to the {get_name()} !"
    
    def calculate(self,a,b):
        """Uses the add function to do addition"""
        return add(a,b)

    def subtract(self,a,b):
        """Just a simple subract function: first - second"""
        return(a-b)
    



from setuptools import setup, find_packages


with open("./README.md", 'r') as f:
    description = f.read()

with open("./requirements.txt", 'r') as f:
    requirements = [req[:-1] for req in f.readlines()]

setup(
    name='rashomon_toolkit',
    version='0.2.1',  # INCREMENT!!
    packages=find_packages(),
    install_requires=requirements,
    long_description=description,
    long_description_content_type="text/markdown",
)
